
package czerwony.kapturek.jbwheel;

import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.util.*;

public class WheelToolDialog extends JDialog
{

    private final static String HELP_HEAD
            = "<HTML><BODY bgcolor=\"#6affa2\"><FONT FACE=\"helvetica,arial\" SIZE=-1>";
    private final static String HELP_FOLLOWBAR
            = "Scrolling will be done according to the scrollbar under cursor."
              + "<P>Works in <I>Component under cursor</I> mode.";
    private final static String HELP_BOTHDIRECTIONS
            = "If scrolling in given direction is not possible the program will try to scroll"
              + "<P>in the other direction."
              + "<P><I>Example:</I> If text is to short to scroll it up/down it will be scrolled horizontally.";
    private final static String HELP_DEBUG
            = "Use this option if you have problems with jbWheel."
              + "<P>See <I>readme.txt</I> for details.";

    private JPanel			panelCenter = new JPanel();
    private JPanel			panelOptions = new JPanel();
    private JPanel			panelButtons = new JPanel();
    private JPanel			panelWhat = new JPanel();
    private JPanel			panelHow = new JPanel();
    private JLabel			labelSensitivity = new JLabel( "Sensitivity:" );
    private JSlider			sliderSensitivity = new JSlider( 0, 100 );
    private JCheckBox		checkReverse = new JCheckBox( "Reverse scrolling direction" );
    private JCheckBox		checkFollowBar = new JCheckBox( "Orientation according to scrollbar under cursor" );
    private JCheckBox		checkBothDirections = new JCheckBox( "Try to scroll in both directions" );
    private JRadioButton	radioScrollCursor = new JRadioButton( "Component under cursor" );
    private JRadioButton	radioScrollFocused = new JRadioButton( "Focused component" );
    private JRadioButton	radioScrollEditor = new JRadioButton( "Editor" );
    private JButton			buttonOK = new JButton( "OK" );
    private JButton			buttonCancel = new JButton( "Cancel" );
    private JTable			tableActions;
    private JButton			buttonAdd = new JButton( "Add" );
    private JButton			buttonDelete = new JButton( "Delete" );
    private JCheckBox       checkDebug = new JCheckBox( "Print debug messages to System.out" );

    private ButtonGroup			group = new ButtonGroup();
    private ActionsTableModel	model = new ActionsTableModel();

    private Config	conf;

    public WheelToolDialog( Frame parent, Config conf )
    {
        super( parent );
        try
        {
            this.conf = conf;
            sliderSensitivity.setValue( conf.sensitivity );
            checkReverse.setSelected( conf.reverse );
            checkFollowBar.setSelected( conf.followBar );
            checkBothDirections.setSelected( conf.bothDirections );
            switch ( conf.mode )
            {
                case Config.MODE_EDITORONLY:
                    radioScrollEditor.setSelected( true );
                    break;
                case Config.MODE_UNDERCURSOR:
                    radioScrollCursor.setSelected( true );
                    break;
                case Config.MODE_FOCUSED:
                    radioScrollFocused.setSelected( true );
                    break;
            }
            group.add( radioScrollCursor );
            group.add( radioScrollEditor );
            group.add( radioScrollFocused );

            model.setActions( conf.actions );
            checkDebug.setSelected(conf.debug);

            initUI();
            initListeners();
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
    }

    private void initUI()
    {
        sliderSensitivity.setMinorTickSpacing( 10 );
        sliderSensitivity.setPaintTicks( true );

        panelOptions.setLayout( new GridBagLayout() );
        GridBagConstraints gbc = new GridBagConstraints( 0, 0, 1, 1, 0, 0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,0,0,0), 0, 0 );
        panelOptions.add( labelSensitivity, gbc );
        gbc.weightx = 1;
        gbc.gridx = 1;
        gbc.gridwidth = 2;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        panelOptions.add( sliderSensitivity, gbc );
        gbc.gridx = 0;
        gbc.gridwidth = 3;
        gbc.gridy++;
        panelOptions.add( checkReverse, gbc );
        gbc.gridy++;
        panelOptions.add( checkFollowBar, gbc );
        gbc.gridy++;
        panelOptions.add( checkBothDirections, gbc);

        panelWhat.setBorder( new TitledBorder( BorderFactory.createEtchedBorder(),"What to scroll"));
        panelWhat.setLayout( new FlowLayout() );
        panelWhat.add( radioScrollCursor, null );
        panelWhat.add( radioScrollFocused, null );
        panelWhat.add( radioScrollEditor, null );

        panelHow.setBorder( new TitledBorder( BorderFactory.createEtchedBorder(),"How to scroll"));
        panelHow.setLayout( new BorderLayout() );
        tableActions = new JTable( model );
        JComboBox cb = new JComboBox();
        for ( int i=0; i<Config.Action.names.length; i++ )
            cb.addItem( Config.Action.names[i] );
        tableActions.getColumn( "Action" ).setCellEditor( new DefaultCellEditor( cb ) );
        TableColumnModel tcm = tableActions.getColumnModel();
        tcm.getColumn(0).setPreferredWidth(40);
        tcm.getColumn(1).setPreferredWidth(40);
        tcm.getColumn(2).setPreferredWidth(80);
        JScrollPane sp = new JScrollPane( tableActions );
        JPanel p = new JPanel( new GridLayout(5,1) );
        p.add( buttonAdd );
        p.add( buttonDelete );
        panelHow.add( p, BorderLayout.EAST );
        panelHow.add( checkDebug, BorderLayout.SOUTH );

        panelButtons.setLayout( new GridLayout( 1, 2 ) );
        panelButtons.add( buttonOK, null );
        panelButtons.add( buttonCancel, null );

        panelCenter.setLayout( new BorderLayout() );
        panelCenter.add( panelWhat, BorderLayout.NORTH );
        panelCenter.add( panelHow, BorderLayout.CENTER );

        Border border = new EmptyBorder(5,5,5,5);
        panelCenter.setBorder(border);
        panelOptions.setBorder(border);
        panelButtons.setBorder(border);

        JComponent cp = (JComponent)getContentPane();
        cp.setLayout( new BorderLayout() );
        cp.add( panelOptions, BorderLayout.NORTH );
        cp.add( panelCenter, BorderLayout.CENTER );
        cp.add( panelButtons, BorderLayout.SOUTH );

        StringBuffer sb = new StringBuffer( Version.NAME );
        sb.append( " (" );
        sb.append( Version.REL );
        sb.append( ")" );
        setTitle( sb.toString() );
        pack();
        panelHow.add( sp, BorderLayout.CENTER );

        checkBothDirections.setToolTipText(HELP_HEAD + HELP_BOTHDIRECTIONS);
        checkFollowBar.setToolTipText(HELP_HEAD + HELP_FOLLOWBAR);
        checkDebug.setToolTipText(HELP_HEAD + HELP_DEBUG);
    }

    private void initListeners()
    {
        buttonOK.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    conf.sensitivity = sliderSensitivity.getValue();
                    conf.reverse = checkReverse.isSelected();
                    conf.followBar = checkFollowBar.isSelected();
                    conf.bothDirections = checkBothDirections.isSelected();
                    if ( radioScrollCursor.isSelected() )
                        conf.mode = Config.MODE_UNDERCURSOR;
                    else if ( radioScrollEditor.isSelected() )
                        conf.mode = Config.MODE_EDITORONLY;
                    else if ( radioScrollFocused.isSelected() )
                        conf.mode = Config.MODE_FOCUSED;
                    conf.actions = model.getActions();
                    conf.debug = checkDebug.isSelected();
                    conf.save();
                    dispose();
                }
            });
        buttonCancel.addActionListener(new java.awt.event.ActionListener()
             {
                public void actionPerformed(ActionEvent e)
                {
                    dispose();
                }
            });
        buttonAdd.addActionListener( new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    model.createRow();
                }
            } );
        buttonDelete.addActionListener( new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    model.deleteRows( tableActions.getSelectedRows() );
                }
            } );
    }

    class ActionsTableModel extends AbstractTableModel
    {
        //private String				c1 = "Alt";
        private String				c2 = "Ctrl";
        private String				c3 = "Shift";
        private String				c4 = "Action";
        private String[]			columnNames = { /*c1,*/ c2, c3, c4 };

        private Vector	data = new Vector();

        void setActions( HashMap a )
        {
            Iterator i = a.keySet().iterator();
            while ( i.hasNext() )
            {
                RowData row = new RowData();
                Integer key = (Integer)i.next();
                Integer v = (Integer)conf.actions.get( key );
                int k = key.intValue();
                //row.alt = (k & Config.MK_ALT)!=0;
                row.ctrl = (k & Config.MK_CONTROL)!=0;
                row.shift = (k & Config.MK_SHIFT)!=0;
                row.action = v.intValue();
                data.add( row );
            }
        }

        HashMap getActions()
        {
            HashMap hm = new HashMap();
            for ( int i=0; i<data.size(); i++ )
            {
                RowData row = (RowData)data.get( i );
                int key = 0;
                //if ( row.alt )
                //	key += Config.MK_ALT;
                if ( row.ctrl )
                    key += Config.MK_CONTROL;
                if ( row.shift )
                    key += Config.MK_SHIFT;
                hm.put( new Integer(key), new Integer(row.action) );
            }
            return hm;
        }

        void createRow()
        {
            data.add( new RowData() );
            fireTableRowsInserted( data.size()-1, data.size()-1 );
        }

        void deleteRows( int[] rows )
        {
            for ( int i=rows.length-1; i>=0; i-- )
            {
                data.remove( rows[i] );
            }
            fireTableRowsDeleted( 0, data.size() );
        }

        public String getColumnName( int i )
        {
            return columnNames[i];
        }

        public int getRowCount()
        {
            return data.size();
        }

        public int getColumnCount()
        {
            return columnNames.length;
        }

        public Class getColumnClass( int c )
        {
            return getValueAt( 0, c ).getClass();
        }

        public boolean isCellEditable( int r, int c )
        {
            return true;
        }

        public Object getValueAt( int r, int c )
        {
            RowData row = (RowData)data.get( r );
            switch ( c )
            {
                //case 0: return new Boolean( row.alt );
                case 0: return new Boolean( row.ctrl );
                case 1: return new Boolean( row.shift );
                case 2: return Config.Action.names[ row.action ];
            }
            return null;
        }

        public void setValueAt( Object v, int r, int c )
        {
            RowData row = (RowData)data.get( r );
            switch ( c )
            {
                //case 0: row.alt = ((Boolean)v).booleanValue();
                //	break;
                case 0: row.ctrl = ((Boolean)v).booleanValue();
                    break;
                case 1: row.shift = ((Boolean)v).booleanValue();
                    break;
                case 2:
                    {
                        for ( int i=0; i<Config.Action.names.length; i++ )
                            if ( Config.Action.names[i].equals( v ) )
                                row.action = i;
                    }
                    break;
            }
        }

        class RowData
        {
            //public boolean	alt;
            public boolean  ctrl;
            public boolean	shift;
            public int		action;
        }
    }
}